package cn.com.infosec.math.ec;

import cn.com.infosec.asn1.x9.X9IntegerConverter;
import java.math.BigInteger;

public abstract class ECPoint {
   private static X9IntegerConverter converter = new X9IntegerConverter();
   ECCurve curve;
   ECFieldElement x;
   ECFieldElement y;
   protected boolean withCompression;
   protected ECMultiplier multiplier = null;
   protected PreCompInfo preCompInfo = null;

   protected ECPoint(ECCurve curve, ECFieldElement x, ECFieldElement y) {
      this.curve = curve;
      this.x = x;
      this.y = y;
   }

   public ECCurve getCurve() {
      return this.curve;
   }

   public ECFieldElement getX() {
      return this.x;
   }

   public ECFieldElement getY() {
      return this.y;
   }

   public boolean isInfinity() {
      return this.x == null && this.y == null;
   }

   public boolean isCompressed() {
      return this.withCompression;
   }

   public boolean equals(Object other) {
      if (other == this) {
         return true;
      } else if (!(other instanceof ECPoint)) {
         return false;
      } else {
         ECPoint other1 = (ECPoint)other;
         if (this.isInfinity()) {
            return other1.isInfinity();
         } else {
            return this.x.equals(other1.x) && this.y.equals(other1.y);
         }
      }
   }

   public int hashCode() {
      return this.isInfinity() ? 0 : this.x.hashCode() ^ this.y.hashCode();
   }

   void setPreCompInfo(PreCompInfo preCompInfo) {
      this.preCompInfo = preCompInfo;
   }

   public abstract byte[] getEncoded();

   public abstract ECPoint add(ECPoint var1);

   public abstract ECPoint subtract(ECPoint var1);

   public abstract ECPoint negate();

   public abstract ECPoint twice();

   synchronized void assertECMultiplier() {
      if (this.multiplier == null) {
         ECPoint var10000 = this;
         FpNafMultiplier var1;
         var1 = new FpNafMultiplier.<init>();
         var10000.multiplier = var1;
      }

   }

   public ECPoint multiply(BigInteger k) {
      if (k.signum() >= 0) {
         if (this.isInfinity()) {
            return this;
         } else if (k.signum() == 0) {
            return this.curve.getInfinity();
         } else {
            this.assertECMultiplier();
            return this.multiplier.multiply(this, k, this.preCompInfo);
         }
      } else {
         throw new IllegalArgumentException("The multiplicator cannot be negative");
      }
   }

   public static class F2m extends ECPoint {
      public F2m(ECCurve curve, ECFieldElement x, ECFieldElement y) {
         this(curve, x, y, false);
      }

      public F2m(ECCurve curve, ECFieldElement x, ECFieldElement y, boolean withCompression) {
         super(curve, x, y);
         if (x != null && y == null || x == null && y != null) {
            throw new IllegalArgumentException("Exactly one of the field elements is null");
         } else {
            if (x != null) {
               ECFieldElement.F2m.checkFieldElements(super.x, super.y);
               if (curve != null) {
                  ECFieldElement.F2m.checkFieldElements(super.x, super.curve.getA());
               }
            }

            super.withCompression = withCompression;
         }
      }

      private static void checkPoints(ECPoint a, ECPoint b) {
         if (!a.curve.equals(b.curve)) {
            throw new IllegalArgumentException("Only points on the same curve can be added or subtracted");
         }
      }

      public byte[] getEncoded() {
         if (this.isInfinity()) {
            return new byte[1];
         } else {
            int var1 = ECPoint.converter.getByteLength(super.x);
            byte[] var2 = ECPoint.converter.integerToBytes(this.getX().toBigInteger(), var1);
            byte[] var3;
            if (super.withCompression) {
               (var3 = new byte[var1 + 1])[0] = 2;
               if (!this.getX().toBigInteger().equals(ECConstants.ZERO) && this.getY().multiply(this.getX().invert()).toBigInteger().testBit(0)) {
                  var3[0] = 3;
               }

               System.arraycopy(var2, 0, var3, 1, var1);
            } else {
               byte[] var10000 = ECPoint.converter.integerToBytes(this.getY().toBigInteger(), var1);
               (var3 = new byte[var1 + var1 + 1])[0] = 4;
               System.arraycopy(var2, 0, var3, 1, var1);
               int var4 = var1 + 1;
               System.arraycopy(var10000, 0, var3, var4, var1);
            }

            return var3;
         }
      }

      public ECPoint add(ECPoint b) {
         checkPoints(this, b);
         return this.addSimple((F2m)b);
      }

      public F2m addSimple(F2m b) {
         if (this.isInfinity()) {
            return b;
         } else if (b.isInfinity()) {
            return this;
         } else {
            F2m var10001 = b;
            ECFieldElement.F2m b1 = (ECFieldElement.F2m)b.getX();
            ECFieldElement.F2m var2 = (ECFieldElement.F2m)var10001.getY();
            if (super.x.equals(b1)) {
               return super.y.equals(var2) ? (F2m)this.twice() : (F2m)super.curve.getInfinity();
            } else {
               ECFieldElement.F2m var10000 = var2 = (ECFieldElement.F2m)super.y.add(var2).divide(super.x.add(b1));
               b1 = (ECFieldElement.F2m)var2.square().add(var2).add(super.x).add(b1).add(super.curve.getA());
               var2 = (ECFieldElement.F2m)var10000.multiply(super.x.add(b1)).add(b1).add(super.y);
               F2m var10002 = this;
               ECCurve var4 = super.curve;
               boolean var3 = var10002.withCompression;
               return new F2m(var4, b1, var2, var3);
            }
         }
      }

      public ECPoint subtract(ECPoint b) {
         checkPoints(this, b);
         return this.subtractSimple((F2m)b);
      }

      public F2m subtractSimple(F2m b) {
         return b.isInfinity() ? this : this.addSimple((F2m)b.negate());
      }

      public ECPoint twice() {
         if (this.isInfinity()) {
            return this;
         } else if (super.x.toBigInteger().signum() == 0) {
            return super.curve.getInfinity();
         } else {
            ECFieldElement var10002 = super.x;
            ECFieldElement.F2m var1;
            ECFieldElement.F2m var2 = (ECFieldElement.F2m)(var1 = (ECFieldElement.F2m)var10002.add(super.y.divide(var10002))).square().add(var1).add(super.curve.getA());
            ECFieldElement var3 = super.curve.fromBigInteger(ECConstants.ONE);
            var1 = (ECFieldElement.F2m)super.x.square().add(var2.multiply(var1.add(var3)));
            F2m var5 = this;
            ECCurve var4 = super.curve;
            boolean var6 = var5.withCompression;
            return new F2m(var4, var2, var1, var6);
         }
      }

      public ECPoint negate() {
         F2m var10002 = this;
         ECCurve var1 = super.curve;
         ECFieldElement var2 = this.getX();
         ECFieldElement var4 = this.getY().add(this.getX());
         boolean var3 = var10002.withCompression;
         return new F2m(var1, var2, var4, var3);
      }

      synchronized void assertECMultiplier() {
         if (super.multiplier == null) {
            F2m var10000;
            if (((ECCurve.F2m)super.curve).isKoblitz()) {
               var10000 = this;
               WTauNafMultiplier var1;
               var1 = new WTauNafMultiplier.<init>();
               var10000.multiplier = var1;
            } else {
               var10000 = this;
               WNafMultiplier var2;
               var2 = new WNafMultiplier.<init>();
               var10000.multiplier = var2;
            }
         }

      }
   }

   public static class Fp extends ECPoint {
      public Fp(ECCurve curve, ECFieldElement x, ECFieldElement y) {
         this(curve, x, y, false);
      }

      public Fp(ECCurve curve, ECFieldElement x, ECFieldElement y, boolean withCompression) {
         super(curve, x, y);
         if ((x == null || y != null) && (x != null || y == null)) {
            super.withCompression = withCompression;
         } else {
            throw new IllegalArgumentException("Exactly one of the field elements is null");
         }
      }

      public byte[] getEncoded() {
         if (this.isInfinity()) {
            return new byte[1];
         } else {
            int var1 = ECPoint.converter.getByteLength(super.x);
            byte[] var10000;
            byte[] var10001;
            byte[] var3;
            int var4;
            byte[] var5;
            if (super.withCompression) {
               byte var7;
               if (this.getY().toBigInteger().testBit(0)) {
                  var7 = 3;
               } else {
                  var7 = 2;
               }

               var10000 = var5 = new byte[(var3 = ECPoint.converter.integerToBytes(this.getX().toBigInteger(), var1)).length + 1];
               var10001 = var3;
               var5[0] = var7;
               var4 = var3.length;
               System.arraycopy(var10001, 0, var5, 1, var4);
               return var10000;
            } else {
               byte[] var2;
               var10000 = var2 = ECPoint.converter.integerToBytes(this.getX().toBigInteger(), var1);
               var3 = ECPoint.converter.integerToBytes(this.getY().toBigInteger(), var1);
               var10000 = var5 = new byte[var10000.length + var3.length + 1];
               var10001 = var3;
               byte[] var10002 = var3;
               var5[0] = 4;
               var4 = var2.length;
               System.arraycopy(var2, 0, var5, 1, var4);
               var4 = var2.length + 1;
               int var6 = var10002.length;
               System.arraycopy(var10001, 0, var5, var4, var6);
               return var10000;
            }
         }
      }

      public ECPoint add(ECPoint b) {
         if (this.isInfinity()) {
            return b;
         } else if (b.isInfinity()) {
            return this;
         } else if (super.x.equals(b.x)) {
            return super.y.equals(b.y) ? this.twice() : super.curve.getInfinity();
         } else {
            ECFieldElement var10000 = b.y.subtract(super.y).divide(b.x.subtract(super.x));
            ECFieldElement b1 = var10000.square().subtract(super.x).subtract(b.x);
            ECFieldElement var2 = var10000.multiply(super.x.subtract(b1)).subtract(super.y);
            return new Fp(super.curve, b1, var2);
         }
      }

      public ECPoint twice() {
         if (this.isInfinity()) {
            return this;
         } else if (super.y.toBigInteger().signum() == 0) {
            return super.curve.getInfinity();
         } else {
            ECFieldElement var1 = super.curve.fromBigInteger(BigInteger.valueOf(2L));
            ECFieldElement var2 = super.curve.fromBigInteger(BigInteger.valueOf(3L));
            ECFieldElement var10000 = super.x.square().multiply(var2).add(super.curve.a).divide(super.y.multiply(var1));
            var1 = var10000.square().subtract(super.x.multiply(var1));
            var2 = var10000.multiply(super.x.subtract(var1)).subtract(super.y);
            Fp var10002 = this;
            ECCurve var4 = super.curve;
            boolean var3 = var10002.withCompression;
            return new Fp(var4, var1, var2, var3);
         }
      }

      public ECPoint subtract(ECPoint b) {
         return (ECPoint)(b.isInfinity() ? this : this.add(b.negate()));
      }

      public ECPoint negate() {
         Fp var10002 = this;
         Fp var10003 = this;
         Fp var10004 = this;
         ECCurve var4 = super.curve;
         ECFieldElement var1 = var10004.x;
         ECFieldElement var2 = var10003.y.negate();
         boolean var3 = var10002.withCompression;
         return new Fp(var4, var1, var2, var3);
      }

      synchronized void assertECMultiplier() {
         if (super.multiplier == null) {
            Fp var10000 = this;
            WNafMultiplier var1;
            var1 = new WNafMultiplier.<init>();
            var10000.multiplier = var1;
         }

      }
   }
}
